using System;
using System.Text;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;

namespace Lexer
{

    public class LexerException : System.Exception
    {
        public LexerException(string msg)
            : base(msg)
        {
        }

    }

    public class Lexer
    {

        protected int position;
        protected char currentCh; // очередной считанный символ
        protected int currentCharValue; // целое значение очередного считанного символа
        protected System.IO.StringReader inputReader;
        protected string inputString;

        public Lexer(string input)
        {
            inputReader = new System.IO.StringReader(input);
            inputString = input;
        }

        public void Error()
        {
            System.Text.StringBuilder o = new System.Text.StringBuilder();
            o.Append(inputString + '\n');
            o.Append(new System.String(' ', position - 1) + "^\n");
            o.AppendFormat("Error in symbol {0}", currentCh);
            throw new LexerException(o.ToString());
        }

        protected void NextCh()
        {
            this.currentCharValue = this.inputReader.Read();
            this.currentCh = (char)currentCharValue;
            this.position += 1;
        }

        public virtual bool Parse()
        {
            return true;
        }
    }

    public class IntLexer : Lexer
    {

        protected System.Text.StringBuilder intString;
        public int parseResult = 0;

        public IntLexer(string input)
            : base(input)
        {
            intString = new System.Text.StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            int sign = 1;

            if (currentCh == '+' || currentCh == '-')
            {
                if (currentCh == '-')
                {
                    sign = -1;
                }
                NextCh();

            }

            if (char.IsDigit(currentCh))
            {
                parseResult = currentCh - '0';
                NextCh();

            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh))
            {
                parseResult *= 10;
                parseResult += currentCh - '0';
                NextCh();

            }


            if (currentCharValue != -1)
            {
                Error();

            }
            parseResult *= sign;
            return true;

        }
    }

    public class IdentLexer : Lexer
    {
        private string parseResult;
        protected StringBuilder builder;

        public string ParseResult
        {
            get { return parseResult; }
        }

        public IdentLexer(string input) : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();

            if (char.IsLetterOrDigit(currentCh))
            {
                parseResult += currentCh;
                NextCh();

            }
            else
            {
                Error();
            }

            while (char.IsLetterOrDigit(currentCh) || currentCh == '_')
            {
                parseResult += currentCh;
                NextCh();

            }
            if (currentCharValue != -1)
            {
                Error();

            }
            return true;
        }

    }

    public class IntNoZeroLexer : IntLexer
    {
        public IntNoZeroLexer(string input)
            : base(input)
        {
        }

        public override bool Parse()
        {

            NextCh();
            int sign = 1;

            if (currentCh == '+' || currentCh == '-')
            {
                if (currentCh == '-')
                {
                    sign = -1;
                }
                NextCh();

            }

            if (char.IsDigit(currentCh))
            {
                if (currentCh == '0')
                {
                    Error();

                }
                else
                {
                    parseResult = currentCh - '0';
                    NextCh();
                }
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh))
            {
                parseResult *= 10;
                parseResult += currentCh - '0';
                NextCh();

            }


            if (currentCharValue != -1)
            {
                Error();

            }
            parseResult *= sign;
            return true;
        }
    }

    public class LetterDigitLexer : Lexer
    {
        protected StringBuilder builder;
        protected string parseResult;

        public string ParseResult
        {
            get { return parseResult; }
        }

        public LetterDigitLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            bool let = false;

            if (char.IsLetter(currentCh))
            {
                parseResult += currentCh;
                let = true;
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsLetterOrDigit(currentCh) )
            {
                if ((char.IsDigit(currentCh) && let) ||
                    (!let && char.IsLetter(currentCh)))
                {
                    parseResult += currentCh;
                    let = !let;
                    NextCh();
                }
                else
                {
                    Error();
                }

            }
            if (currentCharValue != -1)
            {
                Error();

            }
            return true;
        }

    }

    public class LetterListLexer : Lexer
    {
        protected List<char> parseResult;

        public List<char> ParseResult
        {
            get { return parseResult; }
        }

        public LetterListLexer(string input)
            : base(input)
        {
            parseResult = new List<char>();
        }

        public override bool Parse()
        {
            NextCh();
            bool er = false;
            if (char.IsLetter(currentCh))
            {
                parseResult.Add(currentCh);
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsLetter(currentCh) || currentCh == ',' || currentCh == ';')
            {
                if (currentCh == ',' || currentCh == ';')
                {
                    if (er)
                    {
                        Error();
                    }
                    else
                    {
                        er = true;
                        NextCh();
                    }
                }
                else
                {
                    if (!er)
                    {
                        Error();
                    }
                    else
                    {
                        parseResult.Add(currentCh);
                        er = false;
                        NextCh();
                    }
                }

            }


            if (currentCharValue != -1 || er)
            {
                Error();
            }

            return true;
        }
    }

    public class DigitListLexer : Lexer
    {
        protected List<int> parseResult;

        public List<int> ParseResult
        {
            get { return parseResult; }
        }

        public DigitListLexer(string input)
            : base(input)
        {
            parseResult = new List<int>();
        }

        public override bool Parse()
        {
            NextCh();
            bool er = false;
            if (char.IsDigit(currentCh))
            {
                parseResult.Add(currentCh - '0');
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh) || currentCh == ' ')
            {
                if (currentCh != ' ')
                {
                    if (!er)
                    {
                        Error();
                    }
                    else
                    {
                        parseResult.Add(currentCh - '0');
                        er = false;
                        NextCh();
                    }
                }
                else
                {
                    er = true;
                    NextCh();
                }

            }


            if (currentCharValue != -1 || er)
            {
                Error();
            }

            return true;
        }
    }

    public class LetterDigitGroupLexer : Lexer
    {
        protected StringBuilder builder;
        protected string parseResult;

        public string ParseResult
        {
            get { return parseResult; }
        }

        public LetterDigitGroupLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }
        

        public override bool Parse()
        {
            int cLetters = 1;
            int cDigits = 0;

            NextCh();

            if (!char.IsLetter(currentCh))
            {
                Error();
            }

            builder.Append(currentCh);

            NextCh();
            while (char.IsLetterOrDigit(currentCh))
            {
                if (cLetters < 2 && char.IsLetter(currentCh))
                {
                    builder.Append(currentCh);
                    cDigits = 0;
                    cLetters++;
                }
                else if (cDigits < 2 && char.IsDigit(currentCh))
                {
                    builder.Append(currentCh);
                    cDigits++;
                    cLetters = 0;
                }
                else
                    Error();

                NextCh();
            }

            if (currentCharValue != -1 )
            {
                Error();

            }

            if (cDigits > 2|| cLetters > 2)
            {
                Error();
            }

            parseResult = builder.ToString();

            return true;
        }


    }

    public class DoubleLexer : Lexer
    {
        private StringBuilder builder;
        private double parseResult;

        public double ParseResult
        {
            get { return parseResult; }

        }

        public DoubleLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();

            bool point = false;
            if (char.IsDigit(currentCh))
            {
                parseResult = currentCh - '0';
                NextCh();
            }
            else
            {
                Error();
            }

            while (char.IsDigit(currentCh) || currentCh == '.')
            {

                if (currentCh == '.')
                {
                    point = true;
                    NextCh();
                }
                else
                {
                    if (!point)
                    {
                        parseResult *= 10;
                        parseResult += currentCh - '0';
                        NextCh();

                    }
                    else
                    {
                        parseResult += (currentCh - '0') * 0.1;
                        point = false;
                        NextCh();
                    }

                }


            }


            if (currentCharValue != -1 || point)
            {
                Error();

            }
            return true;
        }

    }

    public class StringLexer : Lexer
    {
        private StringBuilder builder;
        private string parseResult;

        public string ParseResult
        {
            get { return parseResult; }

        }

        public StringLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            if (currentCh == '\'')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }            
            while (currentCh != '\'' && currentCh != '\uffff')
            {
                parseResult += currentCh;
                NextCh();
            }
            
            if (currentCh == '\'')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }

            if (currentCharValue != -1)
            {
                Error();
            }

            return true;
        }
    }

    public class CommentLexer : Lexer
    {
        private StringBuilder builder;
        private string parseResult;

        public string ParseResult
        {
            get { return parseResult; }

        }

        public CommentLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
        }

        public override bool Parse()
        {
            NextCh();
            bool star = false;
            if (currentCh == '/')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }
            if (currentCh == '*')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }
            while (currentCh != '/' && currentCh != '\uffff')
            {
                if (currentCh == '*')
                {
                    star = true;
                    parseResult += currentCh;
                    NextCh();
                }
                else
                {
                    parseResult += currentCh;
                    star = false;
                    NextCh();
                }

            }
            if (currentCh == '*' && !star || currentCh != '*' && star)
            {
                if (!star)
                {
                    parseResult += currentCh;
                    NextCh();
                }
            }
            else
            {
                Error();
            }
            if (currentCh == '/')
            {
                parseResult += currentCh;
                NextCh();
            }
            else
            {
                Error();
            }

            if (currentCharValue != -1)
            {
                Error();
            }

            return true;
        }
    }

    public class IdentChainLexer : Lexer
    {
        private StringBuilder builder;
        private List<string> parseResult;

        public List<string> ParseResult
        {
            get { return parseResult; }

        }

        public IdentChainLexer(string input)
            : base(input)
        {
            builder = new StringBuilder();
            parseResult = new List<string>();
        }
        
        
        public override bool Parse()
        {
            NextCh();

            if (char.IsLetter(currentCh))
            {
                builder.Append(currentCh);
                NextCh();
            }
            else
                Error();

            while (char.IsLetterOrDigit(currentCh) || currentCh == '_')
            {
                builder.Append(currentCh);
                NextCh();
            }

            parseResult.Add(builder.ToString());

            if (currentCharValue == -1)
            {
                return true;
            }

            if (currentCh != '.')
            {
                Error();
            }

            while (true)
            {
                NextCh();

                if (char.IsLetter(currentCh) || currentCh == '_')
                {
                    builder.Append(currentCh);
                    NextCh();
                }
                else
                    Error();

                while (char.IsLetterOrDigit(currentCh) || currentCh == '_')
                {
                    builder.Append(currentCh);
                    NextCh();
                }

                parseResult.Add(builder.ToString());

                if (currentCharValue == -1)
                {
                    break;
                }
            }

            return true;
        }

    }

    public class Program
    {
        public static void Main()
        {
            string input = "154216";
            Lexer L = new IntLexer(input);
            try
            {
                L.Parse();
            }
            catch (LexerException e)
            {
                System.Console.WriteLine(e.Message);
            }

        }
    }
}