%using ScannerHelper;
%namespace SimpleScanner

Alpha 	[a-zA-Z_]
Digit   [0-9] 
AlphaDigit {Alpha}|{Digit}
INTNUM  {Digit}+
REALNUM {INTNUM}\.{INTNUM}
ID {Alpha}{AlphaDigit}* 
DotChr [^\r\n]
OneLineCmnt  \/\/{DotChr}*

//     ,    -     Scanner
%{
  public int LexValueInt;
  public double LexValueDouble;
%}

%x COMMENT

%%
{INTNUM} { 
  LexValueInt = int.Parse(yytext);
  return (int)Tok.INUM;
}

{REALNUM} { 
  LexValueDouble = double.Parse(yytext);
  return (int)Tok.RNUM;
}

begin { 
  return (int)Tok.BEGIN;
}

end { 
  return (int)Tok.END;
}

cycle { 
  return (int)Tok.CYCLE;
}

{ID}  { 
  return (int)Tok.ID;
}

":" { 
  return (int)Tok.COLON;
}

":=" { 
  return (int)Tok.ASSIGN;
}

";" { 
  return (int)Tok.SEMICOLON;
}

"{" { 
  BEGIN(COMMENT);
  return (int)Tok.LONGCOMMENT;
}


[^ \r\n] {
	LexError();
	return 0; //  
}

{OneLineCmnt} {
	return (int)Tok.COMMENT;
}

\'[^']*\' {
	return (int)Tok.STRINGAP;
}


<COMMENT> "}" { 
  //    INITIAL
  BEGIN(INITIAL);
}

<COMMENT> begin {
}

<COMMENT> end {
}

<COMMENT> cycle {
}


<COMMENT>{ID} {
  //  ID  
 return (int)Tok.IDCOMMENT; 
}


%%

//        -      Scanner

public void LexError()
{
	Console.WriteLine("({0},{1}):   {2}", yyline, yycol, yytext);
}

public string TokToString(Tok tok)
{
	switch (tok)
	{
		case Tok.ID:
			return tok + " " + yytext;
		case Tok.INUM:
			return tok + " " + LexValueInt;
		case Tok.RNUM:
			return tok + " " + LexValueDouble;
		default:
			return tok + "";
	}
}

